******************************************************
** PROJECT	: NEWFIRM
** TITLE	: EXEC PORTION OF NEWFIRM CODE
** VERSION	: 001
** DATE		: 09/13/2004
** AUTHOR	: PETER MOORE/JOHN GARCIA
** HISTORY	: 07/11/2003 PCM ORIGINAL VERSION.
**			: 07/21/2003     Source aligned to /monsoonhome/aladdin_test.
**			:                Backups in /waveforms/aladdinII.
**			: 07/25/2003 pcm Added fowler sampling + digital averaging
**			: 08/06/2003 pcm Added option to remove reset row strobe and make as normal read
**			: 11/12/2003 pcm Modified to run continous readouts when cmd reg bit set without
**                          requiring a start exposure command to be sent. Removed the PAN
**                          problem delay counter function
**			: 11/13/2003 pcm Added test counter for generating sequences of ctc commands
**			: 02/18/2004 pcm Added cancel at beginning to eliminate stray start exp flag hangs
**                           Coded the pattern memory data into source to exercise asm3 tool,
**                           Added co-add loop counter.
**                      : 04/07/2004 jpg modified for ORION II
**                           a) Alteration of FBIAS levels for ORION II
**                           b) Consolidation of pattern memory for easy changes
**                           c) Additional documentation of patterns in particular
**                           d) INIT subroutine modified to EXPLICITLY set up Fast Biases.
**                           d) Delays need close inspection!!!
**                      : 04/20/04 jpg modified for extra phiC2 pulse for col ref. cell and extra
**                            phiCS pulse to turn off TCend.  Does not trigger ADC for ref. cell.
**                      : 04/20/04 jpg added idle current shutdown during integration.
**                            READ_INIT and READ_END modified.
**                      : 04/21/04  Digital averaging delay loops set for 6 us sampling (jpg).
**                      : 04/22/04  INIT subroutine set up to SELECT all 2048 row/column pairs (jpg).
**                      : 04/22/04  Additional delay (6 ms total) between IidleREF turn on and
**                                  readout start for settling. (jpg)
**                      : 04/23/04  phiDES set for DHE low = ORION chip high in INIT subroutine
**                                  DESELECTION Section (jpg)
**                      : 04/26/04  Select all code section of INIT corrected. (jpg)
**                      : 04/27/04  ADC trig and dig. ave. delay loop added for column ref cell. (jpg)
**                      : 04/29/04  Extended delay between ADC conversions by 20 sequencer clock periods (jpg)
**                      : 04/30/04  Clock and Bias board moved to slot 8 (4th physical). (jpg)
**                      : 05/12/04  Extended width of row/col sysnc pulses and row/col clock underlap. (jpg)
**                      : 05/13/04  Enabled Row Reset Option (new FB pattern, delays) (jpg)
**                      : 05/19/04  VRSTR levels shifted from (0 to -4.0 v) to (-3.60 to -6.5 v) (jpg)
**
**
**                      : 07/13/04  Digital Average loop delay (us) assigned a label "DA_DELAY" for V004
**                                  Currently #DA_DELAY = 2 (jpg).
**
**                      : 08/25/04  DMS 5 added at beginning of Fowler loop as a hopefull cure for incomplete data
**                                  when using Monsoon Engineering Console.
**
**                      : 09/13/04  NOTE: This version DOES NOT have correct Clock and Bias signal map. Board positions
**                                  have been corrected for NEWFIRM chassis.  TEST PURPOSES ONLY.
**
**                      : 09/13/04  Changed so that subroutine INIT is called first time code is run and not during
**                                  normal idle mode.
**
**                      : 09/14/04 Changed for externally controllable delay in milliseconds at end of Fowler loop.
**                                 Loop Register 5 used for frame delay in milliseconds.
**
**                      : 12/12/04 Clock assignments and patterns modified and checked for NEWFIRM (JPG).
**                                 IidleRef switching modified for NEWFIRM
**
**                      : 12/17/04 Fast Bias assignments and patterns modifified and checked for NEWFIRM (JPG)
**                                 General code clean up.
**
**                      : 01/14/05 Switching of IidleRef in READINIT and READ_END disabled to debug code.
**                   
**                      : 01/26/05 Switching of IidleRef in READINIT and READ_END re-enabled.
**
**
** DESCRIPTION : Does read operation for an image size determined by
**             : Rows=#rowloopsx(4rows/loop)x(2images/Fowler pair)
**             : Cols= 72 x 33 for 72 active IR Acq channels and 32 data + 1 reference Orion cols.
**             : Rows=4096 x (#Fowler pairs) for 512 rowloops, ORION
**             : Cols=2376 , ORION with column reference pixels at end for 33 cols and 72 active chan.
**
**             : To use digital averaging, set cntr 3 to the number of averages required
**             : then set bit 14 in the CMD register high (c w 1 0x102 4), then set the
**             : same value loaded to cntr 3 to the IR Acquisition board control register
**             : bits 7 : 2. eg. for an average of 6 + pipeline writes + daughter board
**             : use c w 32 0xfffc 0x1b. for 4 averages use c w 32 0xfffc 0x13, etc.
**             : The only purpose of cntr 3 in this code is to provide sufficient delay
**             : for the adc's to cycle through the required averaging conversions.
**             :
**             : To enable the row reset read option (noise testing), set bit 15 in the CMD reg.
**
**             : Clock HIGH in this code = C/B board ouput HIGH (More +) = logical LOW (0) to ORION
**             : Clock LOW  in this code = C/B board ouput low  (less +) = logical HIGH (1) to ORION
**
** RESOURCES :
** CMD REG
** BIT
**
** 0 => ALWAYS ONE. FORMS THE BRANCH UNCONDITIONAL INSTRUCTION.
** 1 => SYNCHRONIZATION BIT. SET BY EXTERNAL SIGNAL IN SLAVE MODE ELSE INTERNAL EFR REGISTER BIT.
** 2 => INTEGRATION TIMER FLAG. INDICATES THAT INTEGRATION TIME HAS EXPIRED.
** 3 => SET BY THE START EXPOSURE ROUTINE (set by Start Exposure vector)
** 4 => DO FRAME READOUT (I.E. RST READ INTEGRATE READ)
** 5 => LOOP TEST TO TRIGGER CTC STROBES AT CERTAIN PERIODICITY
** 6 => DEBUG FSEH LOOP ON DEBUG LOOP CNTR
** 7 => DEBUG FSOL LOOP ON DEBUG LOOP CNTR
** 8 => DEBUG FSEL LOOP ON DEBUG LOOP CNTR
** 9 => DEBUG INIT LOOP ON DEBUG LOOP CNTR
** (set by direct write to Command Register at MCB addr 0x102)
** 12 => DO RESETS DURING IDLE MODE (command "c w 1 0x102 1")
** 13 => DO CONTINUOUS READOUTS     (command "c w 1 0x102 2")
** 14 => ENABLE DIGITAL AVERAGING   (command "c w 1 0x102 4")
** 15 => ENABLE ROW RESET READS     (command "c w 1 0x102 8")
**
** LOOP CNTR
** REGS
**  0  => ROW COUNTER = 512 USED BY THE READ SUBROUTINE (MCB addr 0x110)
**  2  => FOWLER SAMPLES = 1 TO 16
**  3  => DIGITAL AVERAGES REQUIRED (NOTE, MUST UPDATE THE START.CDL FILE TO CORRECTLY SET ADC'S)
**  4  => CO-ADD LOOP COUNTER.
**  5-12 => INIT loop counters for row/column pair selction/deselection.
** (0x011D) 13 => NUMBER OF CTC STROBES TO GENERATE FOR LOOPTST
** (0x011E) 14 => PERIOD BETWEEN CTC STROBES FOR LOOPTST. UNITS ARE 100NS * SEQUENCER PRESCALER
** (0x011F) 15 => DEBUG COUNTER = USER DISCRETION; NUMBER OF TIMES THE SUBROUTINE IS CALLED (MCB addr 0x112)
**
********************************************************


**
** Constants definitions
**
DEF #KILL_EXPFLG    x00000001	        * Cancel start exposure flag using the EFR register
DEF #STRT_INTCNTR   x00000002		* Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x00000004		* Stop the integration time counter using the EFR register
DEF #RESET		0	        * Mode register constant for board reset operation
DEF #READ		1	        * Mode register constant for read operation
DEF #WRITE16		2		* Mode register constant for 16 bit write operation
DEF #WRITE32		3		* Mode register constant for 32 bit write operation
DEF #ROW_LOOP		0		* Loop register 0 used as row counter. value=2048/4=512
DEF #COL_LOOP		1		* Loop register 1 used as column counter. value=16
DEF #FOWLER_LOOP	2		* Loop register 2 used as Fowler read counter
DEF #DIGAVG_LOOP	3		* Loop register 3 used as digital average counter
DEF #COADD_LOOP		4		* Loop register 4 used as co-add loop counter
DEF #FRAME_DELAY        5               * Loop register 5 used as FRAME_DELAY

DEF #INIT_LOOP2         6               * Loop register 6 used as INIT loop counter (not used for now)
DEF #INIT_LOOP3         7               * Loop register 7 used as INIT loop counter (not used for now)
DEF #INIT_LOOP4         8               * Loop register 8 used as INIT loop counter (not used for now)
DEF #INIT_LOOP5         9               * Loop register 9 used as INIT loop counter (not used for now)
DEF #INTI_LOOP7         10              * Loop register 10 used as INIT loop counter(not used for now)
DEF #INTI_LOOP8         11              * Loop register 11 used as INIT loop counter(not used for now)
DEF #INTI_LOOP9         12              * Loop register 12 used as INIT loop counter(not used for now)

DEF #TST_CTC_CNT	13		* Test routine loop counters
DEF #TST_CTC_PRD	14
DEF #TST_LOOP		15
DEF #DESELECT		0		* select codes for boards
DEF #SELMCB		1		* MCB in slot 1 (1st physical)
DEF #SELCBB		2		* CLK & BIAS Board in slot 2 (2ND physical)
DEF #SELACQ		60		* IR ACQUISITION Boards in slots 4,8,16,32  (3rd, 4th, 5th and 6th)
DEF #CLOCKREG		0		* Clock board address assignements - Clock register
DEF #FBIAS0123		1		* first set of fast biases
DEF #FBIAS4567		2		* second set of fast biases
DEF #ACQ_CTC		0		* Acquisition board addresses - Command to convert register
DEF #DA_DELAY           2               * Digital Average Loop Delay in us.

DEF #PAT_SEG_00 X00                     * Pattern Memory
DEF #PAT_SEG_FB X20
DEF #PAT_SEG_40 X40
DEF #PAT_SEG_60 X60
DEF #PAT_SEG_80 X80
DEF #PAT_SEG_A0 XA0
DEF #PAT_SEG_C0 XC0
DEF #PAT_SEG_E0 XE0
DEF #PAT_SEG_100 X100
DEF #PAT_SEG_120 X120
DEF #PAT_SEG_140 X140
DEF #PAT_SEG_160 X160
DEF #PAT_SEG_180 X180
DEF #PAT_SEG_1A0 X1A0
DEF #PAT_SEG_1C0 X1C0
DEF #PAT_SEG_1E0 X1E0
DEF #PAT_SEG_200 X200

**
** Pattern memory segment definitions
**

ORG PAT #PAT_SEG_00				     * Redundent code !
PAT #KILL_EXPFLG		#KILL_EXPFLG	     * Same label but different contexts
PAT #STRT_INTCNTR		#STRT_INTCNTR	     * The definition earlier: a data value. The label here:
PAT #STOP_INTCNTR		#STOP_INTCNTR	     * a pattern memory address
PAT #IidleOFF1                   x010400AE           * To turn OFF Col Idle current during integration (Bias4).
PAT #IidleON1                    x0104008C           * To turn ON Col Idle current during readout (Bias4).
PAT #IidleOFF2                   x011400AE           * To turn OFF Col Idle current during integration (Bias20).
PAT #IidleON2                    x0114008C           * To turn ON Col Idle current during readout (Bias20).
*
*
*
***********************  Fast Bias Assignment ORION TEST SET *******************************************
*                            (INCLUDED FOR REFERENCE ONLY)                                             *
*                                                                                                      *
* ORG PAT #PAT_SEG_20                                  * Fast Bias 3-0: VROWON,VROWOFF,Spare,NC        *
* PAT #SEG20_PAT01		 x00000000             * FB3= 0.0, FB2= 0.0, FB1=0.0, FB0=0.0          *
* PAT #SEG20_PAT02               x95000000             * FB3=-4.2, FB2= 0.0, FB1=0.0, FB0=0.0          *
*                                                                                                      *
*                                                                                                      *
*                                                                                                      *
*                                                                                                      *
*                                                                                                      *
* ORG PAT #PAT_SEG_40                                  * Fast Bias 7-4: VRTSR,VRSTG,VDDCL,VGGCL        *
* PAT #SEG40_PAT01		  x80808E79            * FB7= -3.60, FB6=-3.60, FB5=-4.00, FB4=-3.40   *
* PAT #SEG40_PAT02                x80E78E79            * FB7= -3.60, FB6=-6.51, FB5=-4.00, FB4=-3.40   *
* PAT #SEG40_PAT03                xE7808E79            * FB7= -6.51, FB6=-3.61, FB5=-4.00, FB4=-3.40   *
*                                                                                                      *
*                                       V_FB = -7.19 x (CODE/255)                                      *
*                                                                                                      *
********************************************************************************************************
*
*
*
*                               Pointers to NEWFIRM Fast Bias Memory Blocks
*
ORG PAT #PAT_SEG_FB
*                       FBIAS0123:          VROWOFF1    VROWOFF2    VROWON1     VROWON2
PAT #FB0_3PAT0 0        *  Val[0]           FB1= 0.00,  FB2= 0.00,  FB3= 0.00,  FB4= 0.00   Row Disabled
PAT #FB0_3PAT1 1        *  Val[1]           FB1= 0.00,  FB2= 0.00,  FB3=-4.20,  FB4=-4.20   Row Enabled
*
*
*                       FBIAS4567:          VRSTG1      VRSTG2      VRSTR1      VRSTR2
PAT #FB4_7PAT0 0        *  Val[0]           FB4=-3.61,  FB5=-3.61,  FB6=-3.61,  FB7=-3.61   Default
PAT #FB4_7PAT1 1        *  Val[1]           FB4=-6.51,  FB5=-6.51,  FB6=-3.61,  FB7=-3.61   Global Reset
PAT #FB4_7PAT2 2        *  Val[2]           FB4=-3.61,  FB5=-3.61,  FB6=-6.51,  FB7=-6.51   Row Reset
*
*
*
*
*
***************************** Clock Assignment NEWFIRM *******************************************************
*
*
*
*                                            Clock                              RD        RD
*                                            Name                        CCCR RROE CCCR RROE
*                                                                        S12S 12ES S12S 12ES
*
*                                            Clock   3322_2222_2222_1111_1111_1100_0000_0000
*                                            Number  1098_7654_3210_9876_5432_1098_7654_3210


ORG PAT #PAT_SEG_60    * Used for INIT ROW Clocks
PAT #SEG60_PAT01		x0000FEFE          * 0000_0000_0000_0000_1111_1110_1111_1110
PAT #SEG60_PAT02		x0000EEEE          * 0000_0000_0000_0000_1110_1110_1110_1110
PAT #SEG60_PAT03                x0000F6F6          * 0000_0000_0000_0000_1111_0110_1111_0110
PAT #SEG60_PAT04                x0000FAFA          * 0000_0000_0000_0000_1111_1010_1111_1010

PAT #SEG60_PAT0A                x0000FFFF          * 0000_0000_0000_0000_1111_1111_1111_1111
PAT #SEG60_PAT0B                x0000EFEF          * 0000_0000_0000_0000_1110_1111_1110_1111
PAT #SEG60_PAT0C                x0000EEEE          * 0000_0000_0000_0000_1110_1110_1110_1110

ORG PAT #PAT_SEG_80    * Used for READINIT Clocks
PAT #SEG80_PAT01		x00006F6F          * 0000_0000_0000_0000_0110_1111_0110_1111
PAT #SEG80_PAT02		x0000EEEE          * 0000_0000_0000_0000_1111_1111_1111_1111
PAT #SEG80_PAT03		x0000F7F7          * 0000_0000_0000_0000_1111_0111_1111_0111

ORG PAT #PAT_SEG_A0    * Used for FSOH clocks
PAT #SEGA0_PAT01		x0000FBFB          * 0000_0000_0000_0000_1111_1011_1111_1011
PAT #SEGA0_PAT02		x00007B7B          * 0000_0000_0000_0000_0111_1011_0111_1011
PAT #SEGA0_PAT03		x0000BBBB          * 0000_0000_0000_0000_1011_1011_1011_1011
PAT #SEGA0_PAT04		x0000DBDB          * 0000_0000_0000_0000_1101_1011_1101_1011
PAT #SEGA0_PAT05		x0000F9F9          * 0000_0000_0000_0000_1111_1001_1111_1001
*
*
*                                            Clock                              RD        RD
*                                            Name                        CCCR RROE CCCR RROE
*                                                                        S12S 12ES S12S 12ES
*
*                                            Clock   3322_2222_2222_1111_1111_1100_0000_0000
*                                            Number  1098_7654_3210_9876_5432_1098_7654_3210
*
ORG PAT #PAT_SEG_C0   * Used for FSEH Clocks
PAT #SEGC0_PAT01		x0000F9F9          * 0000_0000_0000_0000_1111_1001_1111_1001
PAT #SEGC0_PAT02		x00007979          * 0000_0000_0000_0000_0111_1001_0111_1001
PAT #SEGC0_PAT03		x0000B9B9          * 0000_0000_0000_0000_1011_1001_1011_1001
PAT #SEGC0_PAT04		x0000D9D9          * 0000_0000_0000_0000_1101_1001_1101_1001
PAT #SEGC0_PAT05		x0000FFFF          * 0000_0000_0000_0000_1111_1111_1111_1111

ORG PAT #PAT_SEG_E0   * Used for FSOL Clocks
PAT #SEGE0_PAT01		x0000F7F7          * 0000_0000_0000_0000_1111_0111_1111_0111
PAT #SEGE0_PAT02		x00007777          * 0000_0000_0000_0000_0111_0111_0111_0111
PAT #SEGE0_PAT03		x0030B7B7          * 0000_0000_0000_0000_1011_0111_1011_0111
PAT #SEGE0_PAT04		x0000D7D7          * 0000_0000_0000_0000_1101_0111_1101_0111
PAT #SEGE0_PAT05		x0000F5F5          * 0000_0000_0000_0000_1111_0101_1111_0101

ORG PAT #PAT_SEG_100   * Used for FSEL Clocks
PAT #SEG100_PAT01		x0000F5F5          * 0000_0000_0000_0000_1111_0101_1111_0101
PAT #SEG100_PAT02		x00007575          * 0000_0000_0000_0000_0111_0101_0111_0101
PAT #SEG100_PAT03		x0000B5B5          * 0000_0000_0000_0000_1011_0101_1011_0101
PAT #SEG100_PAT04		x0000D5D5          * 0000_0000_0000_0000_1101_0101_1101_0101
PAT #SEG100_PAT05		x0000FFFF          * 0000_0000_0000_0000_1111_1111_1111_1111

ORG PAT #PAT_SEG_120   * Used for READ_END Clocks
PAT #SEG120_PAT01		x00006F6F          * 0000_0000_0000_0000_0110_1111_0110_1111
PAT #SEG120_PAT02		x0000FFFF          * 0000_0000_0000_0000_1111_1111_1111_1111
**
**
**
**
************************************************************************************************************
**
**
** BEGIN CODE SEGMENT
**
**
** EXECUTIVE PORTION OF NEWFIRM SEQUENCER (NewfirmSeqV004)
**
**
	LMR #WRITE32 				* SET 32 BIT WRITE MODE
	LPP #KILL_EXPFLG			* KILL EXTRANOUS START EXPOSURE FLAGS ON SEQUENCER 1st RUN
	LSR #SELMCB 				* WRITE TO THE EFR TO CANCEL ANY STRAY START EXP FLAGS
	LSR #DESELECT 				* CANCEL THE WRITE
#EXEC	CAL #INIT
#EXEC1	JCB #GOBABY 3		        * IF START EXPOSURE FLAG SET
	JCB #RSTLOOP 12			* ELSE IF RESET REQUIRED DURING IDLE
	JCB #READOUT 13			* DEBUG WAVEFORMS. CONTINUOUS READOUT IF BIT 13 IS SET
	JMP #EXEC1			* ELSE JUMP BACK TO EXEC1
#GOBABY	LPP #KILL_EXPFLG 	        * KILL THE START EXP FLAG PATTERN = 0x00000001
	LMR #WRITE32 			* SET 32 BIT WRITE MODE
	LSR #SELMCB 			* WRITE TO THE EFR
	LSR #DESELECT 			* CANCEL THE WRITE
	JCB #READOUT 4 			* NORMAL RST READ INTEGRATE READ
	JCB #LPTST   5 			* SEND A STREAM OF CTC COMMANDS TO THE IR ACQUISITION
	JCB #DBGFSEH 6 			* DEBUG FSEH LOOP
	JCB #DBGFSOL 7 			* DEBUG FSOL LOOP
	JCB #DBGFSEL 8 			* DEBUG FSEL LOOP
	JCB #DBGINIT 9 			* DEBUG INIT LOOP
	JMP #EXEC1			* NO FLAGS SET, GO HOME.
*
#RSTLOOP CAL #RST 			* GO RESET THE ARRAY
        DMS 2
	JCB #EXEC1 0 			* GO BACK TO #EXEC1, DO NOT GO TO #EXEC AND CALL INIT SUBROUTINE
*
* #READOUT LRB #COADD_LOOP	        * START CO-ADD LOOP
#READOUT CAL #RST		 	* RESET THE ARRAY
        DMS 1				* DELAY 1 MILLISEC FOR RESET TO SETTLE
	LPP #STRT_INTCNTR 		* LOAD PATTERN POINTER FOR START INTEGRATION TIMER PATTERN = 0x00000002
	LMR #WRITE32 			* SET 32 BIT WRITE MODE
	LSR #SELMCB 			* WRITE TO THE EFR
	LSR #DESELECT 			* CANCEL THE WRITE
	CAL #READ 			* READOUT THE ARRAY
#INTWAIT JCB #RDOUT2 2 		* WAIT FOR INTEGRATION TIMER
	JCB #INTWAIT 0
#RDOUT2 LPP #STOP_INTCNTR 	* LOAD PATTERN POINTER TO STOP INTEGRATION TIMER PATTERN = 0x00000004
	LMR #WRITE32 			* SET 32 BIT WRITE MODE
	LSR #SELMCB 			* WRITE TO THE EFR
	LSR #DESELECT 			* CANCEL THE WRITE
	CAL #READ 		        * READOUT THE ARRAY
*	LPE				* END OF CO-ADD LOOP
	JCB #READOUT 13 		* CONTINUOUS READOUT IF BIT 13 IS SET ("c w 1 0x102 2")
	JMP #EXEC1	 			* GO BACK TO SLEEP
*
#READ   LRB #FOWLER_LOOP		* LOOP FOR N FOWLER SAMPLES
        DMS 5                          * Delay 5 ms
	CAL #READINIT 			* SETUP THE CLOCKS FOR READOUT
	LRB #ROW_LOOP 			* LOOP N ROW COUNTS (NOMINALLY 512 for ORION)
	CAL #FSOH
	CAL #FSEH
	CAL #FSOL
	CAL #FSEL
	LPE 				* END OF ROW COUNTER LOOP
	CAL #READEND 			* GO BACK TO INTEGRATION STATE
        DMS #FRAME_DELAY                * WAIT FRAME_DELAY MILLISECONDS TO SOLVE SYSTRAN CONSTIPATION
	LPE				* END OF FOWLER LOOP

	RET
*
#LPTST  LMR #WRITE32 		* SET 32 BIT WRITE MODE
	LPP 0				* PATTERN POINTER SET TO 0 (IRRELEVENT REALLY BUT GOOD FORM)
	LDA #ACQ_CTC			* DEVICE ADDRESS SET TO 0
	LRB #TST_CTC_CNT		* LOOP FOR NUMBER OF CTC STROBES REQUESTED
	LSR #SELACQ 			* TRIGGER ADC
	LSR #DESELECT
	LRB #TST_CTC_PRD		* WAIT FOR SPECIFIED NUMBER OF CLOCK CYCLES BETWEEN STROBES
	LPE
	LPE						* END OF TEST LOOP
	JMP #EXEC1
*
#DBGFSOH LRB #TST_LOOP 		* DEBUG LOOP COUNTER
	CAL #FSOH 		* CALL THE SUBROUTINE
	LPE
	JCB #EXEC1 0 		* GO BACK HOME
*
#DBGFSEH LRB #TST_LOOP 		* DEBUG LOOP COUNTER
	CAL #FSEH 		* CALL THE SUBROUTINE
	LPE
	JMP #EXEC1	 	* GO BACK HOME
*
#DBGFSOL LRB #TST_LOOP 		* DEBUG LOOP COUNTER
	CAL #FSOL 		* CALL THE SUBROUTINE
	LPE
	JMP #EXEC1	 	* GO BACK HOME
*
#DBGFSEL LRB #TST_LOOP 		* DEBUG LOOP COUNTER
	CAL #FSEL 		* CALL THE SUBROUTINE
	LPE
	JCB #EXEC1 0 		* GO BACK HOME
#DBGINIT LRB #TST_LOOP 		* DEBUG LOOP COUNTER
	CAL #INIT 		* CALL THE SUBROUTINE
	LPE
	JMP #EXEC1	 	* GO BACK HOME
**
**
**
**
** END EXECUTIVE PORTION OF NEWFIRM SEQUENCER (NewfirmSeqV004)
**
**
**
**
**
**
***************************************************************************************************
*
** SUBROUTINE INIT
** VERSION : 0.1
** DATE : 07/12/2003
** AUTHOR : JOHN GARCIA, GUSTAVO RAHMER
** HISTORY : 07/12/2003 ORIGINAL VERSION.
** DESCRIPTION : Selects/Deselects Row/Col pairs
** RESOURCES : loop registers 5 - 12 at MCB 0x0115 - 0x011C
*************************************************

* EXPLICITLY SET UP FAST BIASES (new)
#INIT   LMR #WRITE32
        LDA #FBIAS0123 				*
	LPP #FB0_3PAT0                          * VROWON, VROWOFF set to high level
	LSR #SELCBB
	LSR #DESELECT
	LDA #FBIAS4567                          *
	LPP #FB4_7PAT0     		        * VRSTG, VRSTR = -3.61 V
	LSR #SELCBB                             *
	LSR #DESELECT
	DUS 10

* SELECT ALL ROW/COLUMN PAIRS PRIOR TO DESELECTING ANY

        LDA #CLOCKREG
	LPP #SEG60_PAT0A			* Clocks: All high
	LSR #SELCBB
	LSR #DESELECT
	DUS 2 					* delay 2us

	LPP #SEG60_PAT0B 			* Clocks: phiRS low (intitialize register)
	LSR #SELCBB
	LSR #DESELECT
        DUS 5                                   * Delay 5 us

        LDA #CLOCKREG
	LPP #SEG60_PAT0C			* Clocks: phiDES low
	LSR #SELCBB
	LSR #DESELECT
	DUS 30 					* delay 30 us

        LDA #CLOCKREG
	LPP #SEG60_PAT0B			* Clocks: phiDES high
	LSR #SELCBB
	LSR #DESELECT
	DUS 5 					* delay 5 us


        LDA #CLOCKREG
	LPP #SEG60_PAT0A			* Clocks: All high
	LSR #SELCBB
	LSR #DESELECT
	DUS 2 					* delay 2us

*************************************
* DESELECTION SECTION (EXPERIMENTAL)
*************************************
*       LDA #CLOCKREG
*      	LPP #SEG60_PAT01			* Clocks: All high, except phiDES
*	LSR #SELCBB                             * PAT 0-4 in SEG60 leave phiDES low
*	LSR #DESELECT
*	DUS 2
*
*
*       LDA #CLOCKREG
*	LPP #SEG60_PAT03			* Clocks: phiR1 low (initialize register)
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 2
*
*       LRB #INIT_LOOP2                         * Clock row register until a deselect needed
*                                               * phiROE is a don't care, always high here.
*                                               * VROWON always high (0.0 v) so no rows in analog
*                                               * section of MUX are enabled.
*       LDA #CLOCKREG                           *
*	LPP #SEG60_PAT01			* Clocks: All high
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 2

*       LDA #CLOCKREG
*	LPP #SEG60_PAT04			* Clocks: phiR2 low
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 20
*
*Could end loop and deselect row/col pair here, then restart loop
*       LDA #CLOCKREG
*	LPP #SEG60_PAT01			* Clocks: All high
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 2

*       LDA #CLOCKREG
*	LPP #SEG60_PAT03			* Clocks: phiR1 low
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 20

*       Could end loop and deselect row/col pair here, then restart loop
*
*        LPE                                     * End of loop
**********************
*END DESELECTION SECTION
**********************
*       LDA #CLOCKREG
*	LPP #SEG60_PAT01			* Clocks: All high
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 2
*
*	LPP #SEG60_PAT02 			* Clocks: phiRS low (turn off tRend)
*	LSR #SELCBB
*	LSR #DESELECT
*       DUS 2
*
*       LDA #CLOCKREG
*	LPP #SEG60_PAT01			* Clocks: All high
*	LSR #SELCBB
*	LSR #DESELECT
*	DUS 20
*
*
*
RET




**************************************************************************************************
* SUBROUTINE RST
*
* Toggles Fast Bias VRSTG between frame readouts
* Comes before READ_INIT
* Comes after READ_END
* Codes only columns I through M of READ_END (formerly RST/INT) spreadsheet
* Assumes Fast Biases and Clocks have been set to
* the proper states by READ_END
*
**************************************************


#RST	LDA #FBIAS4567 				 * POINT TO FBIAS 4567
	LPP #FB4_7PAT1 		                 * VRSTG set to low level, VRSTR to high
	LSR #SELCBB                              * (nominal values)
	LSR #DESELECT
	DUS 20                                   * Delay 20 us, 150 ns for instructions

	LPP #FB4_7PAT0      		         * Set Fast Bias VRSTG to high level
	LSR #SELCBB
	LSR #DESELECT

RET                                              * END SUBROUTINE RST




***************************************************************************************************
*
* SUBROUTINE READINIT
*
* Initializes slow and fast shift registers
* Comes before FSOH on FIRST row readout
* Comes after RST or INT
* Codes only columns I through M of READ_END spreadsheet
* Assumes Fast Biases and Clocks have been set to
* the proper states by RST or INT.

* Turn ON IidleREF1 (Bias04 @ 0x0104)  and IidleREF2 (Bias20 @ 0x0114) during readout only
*
************************************************

#READINIT LMR #WRITE16                         * 16 bit write mode: address and data
	  LPP #IidleON1                        * Turn on IidleREF1 to array1
	  LSR #SELCBB
          LSR #DESELECT
          LPP #IidleON2                        * Turn on IidleREF2 to array2
	  LSR #SELCBB
          LSR #DESELECT
          LMR #WRITE32
          DMS 6

	  LDA #CLOCKREG			       * POINT TO CLOCKS
	  LPP #SEG80_PAT01 		       * Set clocks PhiRS and PhiCS low
	  LSR #SELCBB
	  LSR #DESELECT
	  DSC 40                               * Delay 2100 ns, 100 ns for instructions

	  LPP #SEG80_PAT02 		       * Set clocks PhiRS and PhiCS high
	  LSR #SELCBB
	  LSR #DESELECT
	  DSC 40                               * Delay 2100 ns, 100 ns for instructions

       	  LPP #SEG80_PAT03 	               * Set clock PhiR1 low
	  LSR #SELCBB
	  LSR #DESELECT
	  DSC 40                               * Delay 2100 ns, 100 ns for instructions

	  LPP #SEG80_PAT02 		       * Set clock PhiR1 high
	  LSR #SELCBB
	  LSR #DESELECT
	  DSC 7                                * Delay 450 ns, 100 ns for instructions

RET                                            * End SUBROUTINE READINIT



*************************************************************************************************
*
* SUBROUTINE FSOH
* (FAST SHIFT,ODD ROW, phiR1 HIGH) = Initial State
* Sets up row clocks and Fast Biases
* For column readout
*
***********************************************
#FSOH   LMR #WRITE32

        DSC 40
        LPP #SEGA0_PAT01 		      * CLOCKS: phiR2 LOW
        LDA #CLOCKREG
        LSR #SELCBB
        LSR #DESELECT

        LPP #FB4_7PAT0 			      * VRSTG, VRSTR set to high level
        LDA #FBIAS4567
        LSR #SELCBB
        LSR #DESELECT


        LPP #FB0_3PAT1
        LDA #FBIAS0123			      * VROWOFF set to high level, VROWON to low
        LSR #SELCBB                           * Address fast bias group 1
        LSR #DESELECT

        JCB #FSOH1 15                         * Jump if doing row reset reads
        DSC 6                                 * Delay 300 ns
        JCB #FSOH2 0		              * Skip the row reset sig

#FSOH1	DSC 1                                 * Delay 50 ns, update  FBIAS4567
	LPP #FB4_7PAT2     		      * VRSTR set to low level, VRSTG to high
	LDA #FBIAS4567
	LSR #SELCBB
	LSR #DESELECT


#FSOH2	DSC 40                                * Delay 2000 ns, update the clock pattern
        LDA #CLOCKREG		              * Address the clock register
	LPP #SEGA0_PAT02 		      * CLOCKS : phiCS LOW
	LSR #SELCBB
	LSR #DESELECT
	DSC 20                                * Delay 1000 ns

	LPP #SEGA0_PAT01 		      * CLOCKS : phiCS HIGH
	LSR #SELCBB
	LSR #DESELECT
	DSC 20                                * Delay 1000 ns

	LPP #SEGA0_PAT03 		      * CLOCKS : phiC1 LOW
	LSR #SELCBB
	LSR #DESELECT
	DSC 20                                * Delay 1000 ns

	LPP #SEGA0_PAT01 		      * CLOCKS : phiC1 HIGH, initial clock pattern
	LSR #SELCBB
	LSR #DESELECT
	DSC 80


	LRB #COL_LOOP                       *** Column Clock Loop (16 times)

	   LPP #SEGA0_PAT04 		         * CLOCKS : phiC2 LOW
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120                               * changed from DSC 100
	   LSR #SELACQ 		                 * TRIGGER ADC
	   LSR #DESELECT
	   JCB #FSOH3 14		         * IF DIGITAL AVERAGING SET
	   JMP #FSOH4			         * ELSE CONTINUE WITHOUT DELAY

    #FSOH3 LRB #DIGAVG_LOOP	                 * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					 * END OF DELAY LOOP

    #FSOH4 DSC 7
	   LPP #SEGA0_PAT01 			 * CLOCKS: phiC2 HIGH
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20

	   LPP #SEGA0_PAT03 		         * CLOCKS:phiC1 LOW
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120                               * changed from DSC 100

	   LSR #SELACQ 		                 * TRIGGER ADCS
	   LSR #DESELECT
	   JCB #FSOH5 14		         * IF DIGITAL AVERAGING SET
	   JMP #FSOH6			         * ELSE CONTINUE WITHOUT DELAY

    #FSOH5 LRB #DIGAVG_LOOP	                 * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					 * END OF DELAY LOOP

    #FSOH6 DSC 7
	   LPP #SEGA0_PAT01 			 * CLOCKS: phiC1 HIGH
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20

        LPE                                *** End of Column Clock Loop


	LPP #SEGA0_PAT04 		   *** Extra Column Clock transition for Orion ref cell
	LSR #SELCBB
	LSR #DESELECT                        * CLOCKS : phiC2 LOW
	DSC 120                              * changed from DSC 100

        LSR #SELACQ 		             * TRIGGER ADCs
	LSR #DESELECT
        JCB #FSOH7 14		             * IF DIGITAL AVERAGING SET
	JMP #FSOH8			     * ELSE CONTINUE WITHOUT DELAY

#FSOH7  LRB #DIGAVG_LOOP	             * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	DUS #DA_DELAY
	LPE
#FSOH8  DSC 7				     * END OF DELAY LOOP

	LPP #SEGA0_PAT01 		     * CLOCKS: phiC2 HIGH
	LSR #SELCBB
	LSR #DESELECT
	DSC 12

        LPP #SEGA0_PAT02                     * CLOCKS: phiCS low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20

	LPP #SEGA0_PAT01                     * CLOCKS: phiCS high
	LSR #SELCBB
	LSR #DESELECT                      *** End extra clock transition



	JCB #FSOH9 15                        * Jump if doing row reset reads
	LPP #FB4_7PAT0                          * VRSTG, VRSTR left at high level if normal read
	JMP #FSOH10       
#FSOH9	LPP #FB4_7PAT2 	                     * VRSTR left at low level, VRSTG at high if row reset	             
#FSOH10	LDA #FBIAS4567                       
	LSR #SELCBB
	LSR #DESELECT
	DSC 2

	LPP #FB0_3PAT0
	LDA #FBIAS0123                       * VROWON, VROWOFF set to high level
	LSR #SELCBB                          * VROWON leads VRSTR
	LSR #DESELECT
	DSC 8

	LPP #FB4_7PAT0 		             * VRSTG, VRSTR set to high level
	LDA #FBIAS4567                       
	LSR #SELCBB
	LSR #DESELECT
	DSC 40                               * delay 2000 ns

	LPP #SEGA0_PAT05 		     * CLOCKs : phiROE goes LOW, phiR1 stays HIGH
	LDA #CLOCKREG
	LSR #SELCBB
	LSR #DESELECT

RET                                          * End SUBROUTINE FSOH



*******************************************************************************************
*
* SUBROUTINE FSEH
* (FAST SHIFT, EVEN ROW, phiR1 HIGH) = Initial State
* Sets up row clocks and Fast Biases
* For column readout
*
*************************************************

#FSEH DSC 40 			                * Delay 2000 ns

	LDA #FBIAS0123 				* VROWON, VROWOFF set to high level
	LPP #FB0_3PAT0                          *
	LSR #SELCBB
	LSR #DESELECT
	LDA #FBIAS4567                          
	LPP #FB4_7PAT0 		                * VRSTG, VRSTR set to high level
	LSR #SELCBB                             *
	LSR #DESELECT                           *
	LDA #CLOCKREG
        DSC 40

	LPP #SEGC0_PAT01                        * Clocks: phiROE high to low
	LSR #SELCBB
	LSR #DESELECT

	LDA #FBIAS0123
	LPP #FB0_3PAT1      		        * VROWOFF set to high level, VROWON to low
	LSR #SELCBB
	LSR #DESELECT

	JCB #FSEH1 15		                * Jump if doing row reset reads
	DSC 6
	JMP #FSEH2			        * Skip the row reset sig

#FSEH1	DSC 1 			                * delay 50 ns
	LDA #FBIAS4567
	LPP #FB4_7PAT2      			* Set VRSTR low
	LSR #SELCBB
	LSR #DESELECT

#FSEH2	DSC 40 			                * delay 2000 ns
	LDA #CLOCKREG
	LPP #SEGC0_PAT02 			* Clocks: phiCS low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20

	LPP #SEGC0_PAT01 			* Clocks: phiCS high
	LSR #SELCBB
	LSR #DESELECT
	DSC 20
 				                * delay 175ns
	LPP #SEGC0_PAT03 			* Clocks: phiC1 low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20 				        * delay 175ns
	LPP #SEGC0_PAT01		        * Clocks: phiC1 high
	LSR #SELCBB
	LSR #DESELECT
	DSC 80 				        * delay 4125ns



	LRB #COL_LOOP 		              *** Column Clock Loop (16 times)

	   LPP #SEGC0_PAT04		           * Clocks: phiC2 low
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120 			           * delay 6000ns changed from DSC 100
	   LSR #SELACQ 		                   * trigger CTC on Acq board
	   LSR #DESELECT                           * (by writing to dev addr 0)
	   JCB #FSEH3 14		           * IF DIGITAL AVERAGING SET
	   JMP #FSEH4			           * ELSE CONTINUE WITHOUT DELAY

   #FSEH3  LRB #DIGAVG_LOOP	                   * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					   * END OF DELAY LOOP

   #FSEH4  DSC 7 			           * delay 625ns
	   LPP #SEGC0_PAT01 			   * Clocks: phiC2 high
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20 				   * delay 1000 ns

	   LPP #SEGC0_PAT03 			   * Clocks: phiC1 low
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120 			           * delay 6000ns changed from DSC 100

	   LSR #SELACQ 		                   * trigger CTC on Acq board
	   LSR #DESELECT                           * (by writing to dev addr 0)
	   JCB #FSEH5 14		           * IF DIGITAL AVERAGING SET
	   JMP #FSEH6			           * ELSE CONTINUE WITHOUT DELAY

   #FSEH5  LRB #DIGAVG_LOOP	                   * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					   * END OF DELAY LOOP

   #FSEH6  DSC 7 			           * delay 350
	   LPP #SEGC0_PAT01 			   * Clocks: phiC1 high
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20 				   * delay 1000 ns

        LPE 				      *** End of Column Clock Loop


	LPP #SEGC0_PAT04 	              *** Extra Column Clock transition for Orion ref cell
	LSR #SELCBB
	LSR #DESELECT                           * CLOCKS : phiC2 LOW
	DSC 120                                 * changed from DSC 100

        LSR #SELACQ 		                * trigger CTC on Acq board
	LSR #DESELECT                           * (by writing to dev addr 0)
	JCB #FSEH7 14		                * IF DIGITAL AVERAGING SET
	JMP #FSEH8			        * ELSE CONTINUE WITHOUT DELAY

#FSEH7  LRB #DIGAVG_LOOP	                * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	DUS #DA_DELAY
	LPE					* END OF DELAY LOOP

#FSEH8	DSC 7 			                * delay 625ns

	LPP #SEGC0_PAT01 			* CLOCKS: phiC2 HIGH
	LSR #SELCBB
	LSR #DESELECT
	DSC 12

        LPP #SEGC0_PAT02                        * CLOCKS: phiCS low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20

	LPP #SEGC0_PAT01                        * CLOCKS: phiCS high
	LSR #SELCBB
	LSR #DESELECT                         *** End extra clock transition



	JCB #FSEH9 15                           * Jump if doing row reset reads
	LPP #FB4_7PAT0		                * VRSTG, VRSTR left at high level if normal read
	JMP #FSEH10
#FSEH9	LPP #FB4_7PAT2 		                * VRSTR left at low level, VRSTG left at high if row reset 
#FSEH10	LDA #FBIAS4567
	LSR #SELCBB
	LSR #DESELECT
	DSC 2 				        * delay 425ns

	LDA #FBIAS0123
	LPP #FB0_3PAT0 			        * VROWON, VROWOFF set to high level
	LSR #SELCBB                             * VROWOFF leads VRSTR
	LSR #DESELECT
	DSC 8 				        * delay 775ns

	LDA #FBIAS4567
	LPP #FB4_7PAT0 	                        * VRSTG, VRSTR set to high level
	LSR #SELCBB
	LSR #DESELECT
	DSC 40 				        * delay 2us

	LDA #CLOCKREG 		                * Clocks: All high
	LPP #SEGC0_PAT05
	LSR #SELCBB
	LSR #DESELECT

RET                                             * End of FSEH subroutine



***************************************************************************************************
*
* SUBROUTINE FSOL
* (FAST SHIFT, ODD ROW, phiR1 LOW) = Initial State
* Sets up row clocks and Fast Biases
* For column readout
*
*************************************************

#FSOL   DSC 40                                  * Was LMR #WRITE32

	LPP #SEGE0_PAT01 		        * Clock Registers to Initial State
	LDA #CLOCKREG                           * CLOCK: phiR1 LOW
	LSR #SELCBB
	LSR #DESELECT

	LPP #FB4_7PAT0      			* VRSTG, VRSTR set to high level
	LDA #FBIAS4567                          
	LSR #SELCBB                             *
	LSR #DESELECT                           * Aladdin: VDDCL low, VGGCL high

	LPP #FB0_3PAT1      			* VROWOFF set to high level, VROWON to low
	LDA #FBIAS0123
	LSR #SELCBB
	LSR #DESELECT

	JCB #FSOL1 15		                * Jump if doing row reset reads
	DSC 6
	JMP #FSOL2			        * Skip the row reset sig

#FSOL1	DSC 1                                   * Update FBIAS4567
	LPP #FB4_7PAT2 			        * Set VRSTR low
	LDA #FBIAS4567
	LSR #SELCBB
	LSR #DESELECT

#FSOL2	DSC 40                                  * Update the clock pattern
	LDA #CLOCKREG
	LPP #SEGE0_PAT02 			* CLOCKS : phiCS LOW
	LSR #SELCBB
	LSR #DESELECT
	DSC 20

	LPP #SEGE0_PAT01                        * Initial clock pattern
	LSR #SELCBB                             * CLOCKS : phiCS HIGH
	LSR #DESELECT
	DSC 20

	LPP #SEGE0_PAT03 			* CLOCKS : phiC1 LOW
	LSR #SELCBB
	LSR #DESELECT
	DSC 20

	LPP #SEGE0_PAT01 			* CLOCKS : phiC1 HIGH
	LSR #SELCBB
	LSR #DESELECT
	DSC 80

	LRB #COL_LOOP                         *** Column Clock Loop (16 times)

	   LPP #SEGE0_PAT04 			   * CLOCKS : phiC2 LOW
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120                                 * changed from DSC 100

	   LSR #SELACQ 		                   * TRIGGER ADCs
	   LSR #DESELECT
	   JCB #FSOL3 14		           * IF DIGITAL AVERAGING SET
	   JMP #FSOL4
      			                           * ELSE CONTINUE WITHOUT DELAY
   #FSOL3  LRB #DIGAVG_LOOP	                   * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					   * END OF DELAY LOOP

   #FSOL4  DSC 7
	   LPP #SEGE0_PAT01 			   * CLOCKS : phiC2 HIGH
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20

	   LPP #SEGE0_PAT03 		           * CLOCKS :phiC1 LOW
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120                                 * changed from DSC 100

	   LSR #SELACQ 		                   * TRIGGER ADCs
	   LSR #DESELECT
	   JCB #FSOL5 14		           * IF DIGITAL AVERAGING SET
	   JMP #FSOL6			           * ELSE CONTINUE WITHOUT DELAY

    #FSOL5 LRB #DIGAVG_LOOP                        * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					   * END OF DELAY LOOP

    #FSOL6 DSC 7

	   LPP #SEGE0_PAT01 			   * CLOCKS: phiC1 HIGH
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20

        LPE                                   *** End of Column Clock Loop


	LPP #SEGE0_PAT04 		      *** Extra Column Clock transition for Orion ref cell
	LSR #SELCBB
	LSR #DESELECT                           * CLOCKS : phiC2 LOW
	DSC 120                                 * changed from DSC 100

        LSR #SELACQ 		                * TRIGGER ADCs
	LSR #DESELECT
	JCB #FSOL7 14		                * IF DIGITAL AVERAGING SET
	JMP #FSOL8			        * ELSE CONTINUE WITHOUT DELAY

#FSOL7  LRB #DIGAVG_LOOP                        * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	DUS #DA_DELAY
	LPE					* END OF DELAY LOOP

#FSOL8	DSC 7

	LPP #SEGE0_PAT01 			* CLOCKS: phiC2 HIGH
	LSR #SELCBB
	LSR #DESELECT
	DSC 12

        LPP #SEGE0_PAT02                        * CLOCKS: phiCS low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20

	LPP #SEGE0_PAT01                        * CLOCKS: phiCS high
	LSR #SELCBB
	LSR #DESELECT                         *** End extra clock transition



	JCB #FSOL9 15                           * Jump if doing row reset reads
	LPP #FB4_7PAT0			        * VRSTG, VRSTR left at high level if normal read
	JMP #FSOL10
#FSOL9	LPP #FB4_7PAT2                             * VRSTR left at low level, VRSTG left at high if row reset                       
#FSOL10	LDA #FBIAS4567                          
	LSR #SELCBB
	LSR #DESELECT
	DSC 2

	LPP #FB0_3PAT0  		                * VROWON, VROWOFF set to high level
        LDA #FBIAS0123                          * VROWON leads VRSTR
	LSR #SELCBB
	LSR #DESELECT
	DSC 8

	LPP #FB4_7PAT0    		                * VRSTG, VRSTR set to high level
	LDA #FBIAS4567
	LSR #SELCBB
	LSR #DESELECT
	LSR #SELCBB
	LSR #DESELECT
	DSC 40

	LPP #SEGE0_PAT05 			* CLOCK : phiROE goes LOW, phiR1 stays LOW
	LDA #CLOCKREG
	LSR #SELCBB
	LSR #DESELECT

RET                                             * END OF FSOL SUBROUTINE



******************************************************************************************* 
*
* SUBROUTINE FSEL
* (FAST SHIFT, EVEN ROW, phiR1 LOW) = Initial State
* Sets up row clocks and Fast Biases
* For column readout
*
*************************************************

#FSEL   DSC 40			                * delay 2000 ns

	LDA #FBIAS0123 				* VROWON, VROWOFF set to high level
	LPP #FB0_3PAT0
	LSR #SELCBB
	LSR #DESELECT

	LDA #FBIAS4567			        * VRSTR,VRSTG set to high level
	LPP #FB4_7PAT0                             *
	LSR #SELCBB                             *
	LSR #DESELECT

	LDA #CLOCKREG
	LPP #SEG100_PAT01                       * Clocks: phiROE, phiR1 low
	LSR #SELCBB
	LSR #DESELECT

	LDA #FBIAS0123
	LPP #FB0_3PAT1                             * VROWON high to low level
	LSR #SELCBB                             * VROWOFF stays high
	LSR #DESELECT
	
	JCB #FSEL1 15		                * Jump if doing row reset reads
	DSC 6
	JMP #FSEL2			        * Skip the row reset sig

#FSEL1	DSC 1 			                * delay 50 ns
	LDA #FBIAS4567
	LPP #FB4_7PAT2                             * Set VRSTR low
	LSR #SELCBB
	LSR #DESELECT
#FSEL2	DSC 40 			                * delay 1000ns

	LDA #CLOCKREG
	LPP #SEG100_PAT02 			* Clocks: phiCS low
	LSR #SELCBB
	LSR #DESELECT

	DSC 20				        * delay 1000 ns
	LPP #SEG100_PAT01       	        * Clocks: phiCS high
	LSR #SELCBB
	LSR #DESELECT
	DSC 20 				        * delay 1000 ns

	LPP #SEG100_PAT03		        * Clocks: phiC1 low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20 				        * delay 1000 ns

	LPP #SEG100_PAT01		        * Clocks: phiC1 high
	LSR #SELCBB
	LSR #DESELECT
	DSC 80 				        * delay 4000 ns

	LRB #COL_LOOP 		              *** Column Clock Loop (16 times)

	   LPP #SEG100_PAT04                       * Clocks: phiC2 low
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120 			           * delay 6000ns changed from 5000ns

	   LSR #SELACQ 		                   * trigger CTC on Acq board
	   LSR #DESELECT                           * (by writing to dev addr 0)
	   JCB #FSEL3 14		           * IF DIGITAL AVERAGING SET
	   JCB #FSEL4 0		                   * ELSE CONTINUE WITHOUT DELAY

    #FSEL3 LRB #DIGAVG_LOOP                        * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					   * END OF DELAY LOOP

    #FSEL4 DSC 7 			           * delay 350 ns
	   LPP #SEG100_PAT01		           * Clocks: phiC2 high
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 20 				   * delay 1000 ns

	   LPP #SEG100_PAT03		           * Clocks: phiC1 low
	   LSR #SELCBB
	   LSR #DESELECT
	   DSC 120 			           * delay 6000ns changed from 5000ns

	   LSR #SELACQ 		                   * trigger CTC on Acq board
           LSR #DESELECT                           * (by writing to dev addr 0)
	   JCB #FSEL5 14		           * IF DIGITAL AVERAGING SET
	   JMP #FSEL6			           * ELSE CONTINUE WITHOUT DELAY

    #FSEL5 LRB #DIGAVG_LOOP                        * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	   DUS #DA_DELAY
	   LPE					   * END OF DELAY LOOP

    #FSEL6 DSC 9 			           * delay 450 ns
	  LPP #SEG100_PAT01		           * Clocks: phiC1 high
	  LSR #SELCBB
	  LSR #DESELECT
	  DSC 20 				   * delay 1000 ns

       LPE 				      *** End of Column Clock Loop


	LPP #SEG100_PAT04 		      *** Extra Column Clock transition for Orion ref cell
	LSR #SELCBB                             * CLOCKS : phiC2 LOW
	LSR #DESELECT
	DSC 120                                 * changed from DSC 100
        LSR #SELACQ 		                * trigger CTC on Acq board

        LSR #DESELECT                           * (by writing to dev addr 0)
	JCB #FSEL7 14		                * IF DIGITAL AVERAGING SET
	JMP #FSEL8			        * ELSE CONTINUE WITHOUT DELAY

#FSEL7  LRB #DIGAVG_LOOP                        * DELAY FOR #DA_DELAY us / DIGITAL AVERAGE
	DUS #DA_DELAY
	LPE					* END OF DELAY LOOP

#FSEL8	DSC 9 			                * delay 450 ns
	LPP #SEG100_PAT01 			* CLOCKS: phiC2 HIGH
	LSR #SELCBB
	LSR #DESELECT
	DSC 12                                  * Delay 600 ns

        LPP #SEG100_PAT02                       * CLOCKS: phiCS low
	LSR #SELCBB
	LSR #DESELECT
	DSC 20                                  * Delay 1000 ns

	LPP #SEG100_PAT01                       * CLOCKS: phiCS high
	LSR #SELCBB
	LSR #DESELECT                         *** End extra clock transition
                             

	JCB #FSEL9 15                           * jump if doing row resets
	LPP #FB4_7PAT0     		        * Set VRSTR, VRSTG to high level
	JMP #FSEL10

#FSEL9	LPP #FB4_7PAT2	                        * VRSTR set to low level
#FSEL10	LDA #FBIAS4567                          * VRSTG stays at high level   
	LSR #SELCBB
	LSR #DESELECT
	DSC 2 				        * delay 100 ns

	LDA #FBIAS0123
	LPP #FB0_3PAT0		                * VROWON low to high level
	LSR #SELCBB                             * VROWOFF stays high
	LSR #DESELECT
	DSC 8 				        * delay 400 ns

	LDA #FBIAS4567
	LPP #FB4_7PAT0		                * Set VRSTR to high level
	LSR #SELCBB
	LSR #DESELECT
	DSC 40 				        * delay 2000 ns
	LDA #CLOCKREG
	LPP #SEG100_PAT05 			* Clocks: phiR1, phiROE high
	LSR #SELCBB
	LSR #DESELECT


RET                                             * End of FSEL subroutine



**************************************************************************************
* 
* SUBROUTINE READ_END
*
* Toggles PhiSS and PhiFS at end of frame readout
* Comes before INT or RST
* Comes after FSEL
*
* Assumes Fast Biases and Clocks have been set to
* the proper states by FSEL
*
**************************************************

#READEND LMR #WRITE32
  	 LDA #CLOCKREG
  	 LPP #SEG120_PAT01                       * Set clocks phiRS and phiCS low 			 
  	 LSR #SELCBB
  	 LSR #DESELECT
  	 DSC 40                                  

  	 LPP #SEG120_PAT02                       * Set clocks phiRS and phiFS high			 
  	 LSR #SELCBB
  	 LSR #DESELECT


	 LMR #WRITE16                            * Change to 16 bit write mode
	 LPP #IidleOFF1                          * Turn OFF IidleREF except during readout              
	 LSR #SELCBB                             * (Bias04 @ 0x0104 and Bias20 @ 0x0114)
         LSR #DESELECT
         LPP #IidleOFF2
	 LSR #SELCBB
         LSR #DESELECT
         LMR #WRITE32                            * Change back to 32 bit write mode
         DMS 2

RET                                              * END READ_END


